﻿Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Xml
Imports System.Xml.Serialization
Imports Syncfusion.UI.Xaml.Diagram

Public Class Serializers

    Public Shared Function ObjectToBytes(ByVal Objdata As Object, ByVal DataType As Type) As Byte()
        Return System.Text.Encoding.UTF8.GetBytes(ObjectToXML(Objdata, DataType))
    End Function

    Public Shared Function StreamToObject(ByVal objStream As IO.Stream, ByVal DataType As Type) As Object
        Try
            Dim formatter As New Xml.Serialization.XmlSerializer(DataType)
            Return formatter.Deserialize(objStream)
        Catch ex As Exception
            MsgBox("There was an exception while trying to convert stream to object. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return Nothing
    End Function

    Public Shared Function ObjectToBinary(ByVal Obj As Object) As Byte()
        Try
            Dim stream As New MemoryStream()
            Dim formatter As New BinaryFormatter()
            formatter.Serialize(stream, Obj)
            Return stream.ToArray()
        Catch ex As Exception
            MsgBox("There was an exception while trying to convert object to byte array. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return Nothing
    End Function

    Public Shared Function BytesToObject(ByVal Obj As Byte()) As Object
        Try
            Dim stream As New MemoryStream(Obj)
            Dim formatter As New BinaryFormatter()
            Return formatter.Deserialize(stream)
        Catch ex As Exception
            MsgBox("There was an exception while trying to convert object to byte array. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return Nothing
    End Function

    Public Shared Function ObjectToBinaryFile(ByVal data As Object, ByVal Filename As String) As Boolean
        Try
            Dim file As New FileInfo(Filename)
            If Not file.Directory.Exists Then
                file.Directory.Create()
            Else
                If file.Exists() = True Then
                    file.Delete()
                End If
            End If

            Dim fs As FileStream = Nothing
            'Dim fs As FileStream = IO.File.OpenWrite(Filename)
            Try
                fs = New FileStream(Filename, FileMode.OpenOrCreate)
                Dim stream As New MemoryStream()
                Dim bf As New Runtime.Serialization.Formatters.Binary.BinaryFormatter()
                bf.AssemblyFormat = Runtime.Serialization.Formatters.FormatterAssemblyStyle.Full
                bf.TypeFormat = Runtime.Serialization.Formatters.FormatterTypeStyle.TypesWhenNeeded
                bf.Serialize(fs, data)
            Catch ex As Exception
                MsgBox("There was an exception while trying to convert object to binary file. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
            Finally
                If fs IsNot Nothing Then
                    fs.Close()
                End If
            End Try
            Return True
        Catch ex As Exception
            MsgBox("There was an exception while trying to convert object to binary file. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return False
    End Function

    Public Shared Function BinaryFileToObject(ByVal Filename As String) As Object
        Try
            If File.Exists(Filename) Then
                'Dim fs As New FileStream(Filename, FileMode.Open)
                Dim fs As FileStream = Nothing
                Try
                    fs = IO.File.OpenRead(Filename)
                    Dim bf As New Runtime.Serialization.Formatters.Binary.BinaryFormatter()
                    bf.AssemblyFormat = Runtime.Serialization.Formatters.FormatterAssemblyStyle.Full
                    bf.TypeFormat = Runtime.Serialization.Formatters.FormatterTypeStyle.TypesWhenNeeded
                    Dim obj As Object = bf.Deserialize(fs)
                    Return obj
                Catch ex As Exception
                    MsgBox("There was an exception while trying to convert binary file to object. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
                Finally
                    If fs IsNot Nothing Then
                        fs.Close()
                    End If
                End Try
            Else
                MsgBox("There is no file found with the name '" & Filename & "' to convert to object.")
            End If
        Catch ex As Exception
            MsgBox("There was an exception while trying to convert binary file to object. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return Nothing
    End Function

    Public Shared Function ObjectToXMLFile(ByVal Data As Object, ByVal DataType As Type, ByVal XmlFilePath As String, ParamArray eXtraTypes() As Type) As Boolean




        Dim xmldoc As New Xml.XmlDocument
        Dim xmlSerializer As New Xml.Serialization.XmlSerializer(DataType, Ignore.GetTypes(), eXtraTypes, Nothing, Nothing)

        Using xmlStream As New MemoryStream()
            xmlSerializer.Serialize(xmlStream, Data)
            xmlStream.Position = 0
            xmldoc.Load(xmlStream)
            xmldoc.Save(XmlFilePath)
        End Using
        Return True
    End Function

    Public Shared Function XMLFileToObject(ByVal XmlFilePath As String, ByVal DataType As Type, ParamArray eXtraTypes() As Type) As Object
        Try
            Dim value As Object = Nothing
            If My.Computer.FileSystem.FileExists(XmlFilePath) Then
                Dim doc As New System.Xml.XmlDocument()
                doc.Load(XmlFilePath)
                Dim reader As New System.IO.StringReader(doc.InnerXml)
                Dim xmlSerializer As New Serialization.XmlSerializer(DataType, Ignore.GetTypes(), eXtraTypes, Nothing, Nothing)
                'AddHandler xmlSerializer.UnknownNode, Sub(sender As Object, e As XmlNodeEventArgs)
                '                                          Dim node = e.Text
                '                                      End Sub
                value = xmlSerializer.Deserialize(reader)
                reader.Close()
            End If
            Return value

        Catch ex As Exception
            Dim strMsg As String = ex.Message
            If ex.InnerException IsNot Nothing Then
                strMsg &= Environment.NewLine & ex.InnerException.Message
            End If
            MsgBox(strMsg)
            'Error while parsing..
        End Try
        Return Nothing
    End Function

    Public Shared Function ObjectToXML(ByVal Data As Object, ByVal DataType As Type, ParamArray eXtraTypes() As Type) As String
        Dim xmldoc As New Xml.XmlDocument
        Dim xmlSerializer As New Xml.Serialization.XmlSerializer(DataType, eXtraTypes)

        Using xmlStream As New MemoryStream()
            xmlSerializer.Serialize(xmlStream, Data)
            xmlStream.Position = 0
            xmldoc.Load(xmlStream)
        End Using
        Return xmldoc.InnerXml
    End Function

    Public Shared Function XmlToObject(ByVal Xml As String, ByVal DataType As Type, ParamArray eXtraTypes() As Type) As Object
        Dim value As Object = Nothing
        Dim reader As New System.IO.StringReader(Xml)
        Dim xmlSerializer As New Serialization.XmlSerializer(DataType, eXtraTypes)
        value = xmlSerializer.Deserialize(reader)
        reader.Close()
        Return value
    End Function

    Public Shared Function ProcessResponse(ByVal strResponse As String)
        Try
            strResponse = strResponse.Replace("<string xmlns=""http://schemas.microsoft.com/2003/10/Serialization/"">", "")
            strResponse = strResponse.Replace("<string xmlns=""http://schemas.microsoft.com/2003/10/Serialization/"" />", "")  ' Added this line after sending build 5.0.2.
            strResponse = strResponse.Replace("</string>", "")
            strResponse = strResponse.Replace("&gt;", ">")
            Return strResponse.Replace("&lt;", "<")
        Catch ex As Exception
            MsgBox("There was an exception while trying to process the response. Exception: " & ex.Message & " | Stacktrace: " & ex.StackTrace)
        End Try
        Return strResponse
    End Function

End Class
